<?php

namespace Drupal\field_fallback\Plugin\FieldFallbackConverter;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\field\FieldConfigInterface;
use Drupal\field_fallback\Plugin\FieldFallbackConverterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The Paragraphs Summary field fallback converter.
 *
 * This converter converts paragraphs to a summary using the
 * paragraphs_summary_token module.
 *
 * @FieldFallbackConverter(
 *   id = "paragraphs_summary",
 *   label = @Translation("Paragraphs summary"),
 *   source = {"entity_reference_revisions"},
 *   target = {"text_long"},
 *   weight = 0
 * )
 */
class ParagraphsSummaryFieldFallbackConverter extends FieldFallbackConverterBase implements ContainerFactoryPluginInterface {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, string $plugin_id, array $plugin_definition, ModuleHandlerInterface $module_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function convert(FieldItemListInterface $field) {
    $configuration = $this->getConfiguration();
    return [
      'value' => \Drupal::service('paragraphs_summary_token.summary_builder')->build($field, $configuration['trim_length']),
      'format' => filter_default_format(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + ['trim_length' => 300];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $configuration = $this->getConfiguration();

    $form['trim_length'] = [
      '#title' => t('Trimmed limit'),
      '#type' => 'number',
      '#field_suffix' => t('characters'),
      '#default_value' => $configuration['trim_length'],
      '#min' => 1,
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(FieldConfigInterface $target_field, FieldConfigInterface $source_field) {
    return $source_field->getSetting('target_type') === 'paragraph' && $this->moduleHandler->moduleExists('paragraphs_summary_token');
  }

}
